import pandas as pd


def join_tracking_events(tracking_records_df: pd.DataFrame, events_df: pd.DataFrame):
    return pd.merge(
        events_df,
        tracking_records_df[["id", "shipment_id"]],
        left_on="tracking_record_id",
        right_on="id",
        how="inner",
    )



import pandas as pd
from ovretl.computed_quotations_utils.add_missing_categories import cq_missing_categories, add_missing_categories
from pandas.util.testing import assert_frame_equal


def test_cq_missing_categories_1():
    cq_df = pd.DataFrame(
        data={
            "category": ["freight", "freight", "departure_fees", "departure_fees"],
            "kronos_selected": [False, True, False, False],
            "selected": [False, False, False, False],
        }
    )
    assert cq_missing_categories(cq_df) == 1


def test_cq_missing_categories_2():
    cq_df = pd.DataFrame(
        data={
            "category": ["freight", "freight", "departure_fees", "departure_truck_freight"],
            "kronos_selected": [False, False, False, False],
            "selected": [False, True, True, False],
        }
    )
    assert cq_missing_categories(cq_df) == 1


def test_add_missing_categories():
    cq_df = pd.DataFrame(
        data={
            "category": ["freight", "freight", "departure_fees", "departure_fees", "freight"],
            "kronos_selected": [False, True, False, False, True],
            "selected": [False, False, False, False, False],
            "computed_quotation_id": [1, 1, 1, 1, 2],
            "proposition_id": [0, 0, 0, 0, 1],
        }
    )
    result_should_be = pd.DataFrame(
        data={
            "category": ["freight", "freight", "departure_fees", "departure_fees", "freight"],
            "kronos_selected": [False, True, False, False, True],
            "selected": [False, False, False, False, False],
            "computed_quotation_id": [1, 1, 1, 1, 2],
            "proposition_id": [0, 0, 0, 0, 1],
            "missing_categories": [1, 1, 1, 1, 0],
            "full": [False, False, False, False, True],
        }
    )
    result = add_missing_categories(cq_df)
    assert_frame_equal(result, result_should_be)



import pandas as pd

from ovretl.prices_utils.sum_category_price import select_category_price


# def calculate_shipment_prices(shipment: pd.Series, df_prices: pd.DataFrame) -> pd.Series:
#     if df_prices is None or len(df_prices) == 0:
#         return shipment
#     df_prices = df_prices.groupby("category").sum()
#     shipment["departure_truck_freight_price"] = select_category_price(
#         df_prices, "departure_truck_freight", "price_in_eur"
#     )
#     shipment["departure_fees_price"] = select_category_price(df_prices, "departure_fees", "price_in_eur")
#     shipment["freight_price"] = select_category_price(df_prices, "freight", "price_in_eur")
#     shipment["arrival_fees_price"] = select_category_price(df_prices, "arrival_fees", "price_in_eur")
#     shipment["arrival_truck_freight_price"] = select_category_price(df_prices, "arrival_truck_freight", "price_in_eur")
#     shipment["insurance_price"] = select_category_price(df_prices, "insurance", "price_in_eur")
#     shipment["turnover"] = (
#         shipment["departure_truck_freight_price"]
#         + shipment["departure_fees_price"]
#         + shipment["freight_price"]
#         + shipment["arrival_fees_price"]
#         + shipment["arrival_truck_freight_price"]
#         + shipment["insurance_price"]
#     )
#     shipment["customs_price"] = select_category_price(df_prices, "customs", "price_in_eur")
#     shipment["other_price"] = select_category_price(df_prices, "other", "price_in_eur")
#     shipment["vat_price"] = df_prices["vat_price_in_eur"].sum()
#     return shipment
#
#
# def calculate_margins(shipment: pd.Series, df_prices: pd.DataFrame, df_prices_initial: pd.DataFrame) -> pd.Series:
#     if df_prices is None or len(df_prices) == 0 or df_prices_initial is None or len(df_prices_initial) == 0:
#         return shipment
#     df_prices = df_prices.groupby("category").sum()
#     df_prices_initial = df_prices_initial.groupby("category").sum()
#
#     shipment["initial_margin_without_insurance"] = (
#         df_prices_initial["margin_price_in_eur"].sum()
#         - select_category_price(df_prices_initial, key="margin_price_in_eur", category="insurance")
#         - select_category_price(df_prices_initial, key="margin_price_in_eur", category="customs")
#         - select_category_price(df_prices_initial, key="margin_price_in_eur", category="other")
#     )
#     shipment["initial_margin_insurance"] = select_category_price(
#         df_prices_initial, key="margin_price_in_eur", category="insurance"
#     )
#     shipment["margin_without_insurance"] = (
#         df_prices["margin_price_in_eur"].sum()
#         - select_category_price(df_prices, key="margin_price_in_eur", category="insurance")
#         - select_category_price(df_prices, key="margin_price_in_eur", category="customs")
#         - select_category_price(df_prices, key="margin_price_in_eur", category="other")
#     )
#     shipment["margin_insurance"] = select_category_price(df_prices, key="margin_price_in_eur", category="insurance")
#     return shipment
#
#
# def calculate_shipment_purchase_prices(shipment: pd.Series, df_prices: pd.DataFrame) -> pd.Series:
#     if df_prices is None or len(df_prices) == 0:
#         return shipment
#     df_prices = df_prices.groupby("category").sum()
#     shipment["departure_truck_freight_purchase_price"] = select_category_price(
#         df_prices, category="departure_truck_freight", key="purchase_price_in_eur"
#     )
#     shipment["departure_fees_purchase_price"] = select_category_price(
#         df_prices, category="departure_fees", key="purchase_price_in_eur"
#     )
#     shipment["freight_purchase_price"] = select_category_price(
#         df_prices, category="freight", key="purchase_price_in_eur"
#     )
#     shipment["arrival_fees_purchase_price"] = select_category_price(
#         df_prices, category="arrival_fees", key="purchase_price_in_eur"
#     )
#     shipment["arrival_truck_freight_purchase_price"] = select_category_price(
#         df_prices, category="arrival_truck_freight", key="purchase_price_in_eur"
#     )
#     shipment["insurance_purchase_price"] = select_category_price(
#         df_prices, category="insurance", key="purchase_price_in_eur"
#     )
#     return shipment
#
#
# def add_prices_to_shipments(shipment: pd.Series, find_most_relevant_prices) -> pd.Series:
#     df_prices, prices_origin = find_most_relevant_prices(shipment["shipment_id"], shipment["proposition_id"])
#     df_prices_initial, initial_prices_origin = find_most_relevant_prices(
#         shipment["shipment_id"], shipment["proposition_id"], initial_proposition=True
#     )
#     shipment = calculate_shipment_prices(shipment=shipment, df_prices=df_prices)
#     shipment = calculate_margins(shipment=shipment, df_prices=df_prices, df_prices_initial=df_prices_initial)
#     shipment = calculate_shipment_purchase_prices(shipment=shipment, df_prices=df_prices)
#     shipment["prices_origin"] = prices_origin
#     return shipment
from ovretl.prices_utils.sum_multiple_billings_prices import sum_multiple_billings_prices


def add_initial_category_prices_to_shipments(
    shipments_with_prices: pd.DataFrame, prices_propositions_by_category_df: pd.DataFrame
) -> pd.DataFrame:
    initial_prices = (
        prices_propositions_by_category_df.drop(["count"], axis=1, errors="ignore")
        .dropna(subset=["margin_price_in_eur", "proposition_id"], how="any")
        .rename(
            columns={"margin_price_in_eur": "initial_margin_price_in_eur", "proposition_id": "initial_proposition_id"}
        )
    )

    shipments_with_prices = pd.merge(
        shipments_with_prices,
        initial_prices[["initial_proposition_id", "category", "initial_margin_price_in_eur"]],
        on=("initial_proposition_id", "category"),
        how="left",
    )
    return shipments_with_prices


def add_category_prices_to_shipments(
    shipments_with_prices_ids: pd.Series,
    prices_final_check_by_category_df: pd.DataFrame,
    prices_billings_by_category_df: pd.DataFrame,
    prices_propositions_by_category_df: pd.DataFrame,
) -> pd.DataFrame:
    shipments_with_prices = pd.merge(
        shipments_with_prices_ids,
        prices_final_check_by_category_df.dropna(subset=["price_in_eur", "final_check_id"], how="any"),
        on="final_check_id",
        how="left",
    )

    shipments_with_prices_full = shipments_with_prices[~shipments_with_prices["price_in_eur"].isnull()]
    shipments_with_prices_uncomplete = shipments_with_prices[shipments_with_prices["price_in_eur"].isnull()].drop(
        [
            "category",
            "price_in_eur",
            "vat_price_in_eur",
            "margin_price_in_eur",
            "purchase_price_in_eur",
            "prices_origin",
        ],
        axis=1,
    )

    shipments_with_prices_uncomplete = pd.merge(
        shipments_with_prices_uncomplete,
        prices_billings_by_category_df.dropna(subset=["price_in_eur", "billing_id"], how="any"),
        on="billing_id",
        how="left",
    )

    shipments_with_prices = pd.concat([shipments_with_prices_full, shipments_with_prices_uncomplete], sort=False)

    shipments_with_prices_full = shipments_with_prices[~shipments_with_prices["price_in_eur"].isnull()]
    shipments_with_prices_uncomplete = shipments_with_prices[shipments_with_prices["price_in_eur"].isnull()].drop(
        [
            "category",
            "price_in_eur",
            "vat_price_in_eur",
            "margin_price_in_eur",
            "purchase_price_in_eur",
            "prices_origin",
        ],
        axis=1,
    )

    shipments_with_prices_uncomplete = pd.merge(
        shipments_with_prices_uncomplete,
        prices_propositions_by_category_df.dropna(subset=["price_in_eur", "proposition_id"], how="any"),
        on="proposition_id",
        how="left",
    )

    shipments_with_prices = pd.concat([shipments_with_prices_full, shipments_with_prices_uncomplete], sort=False)

    shipments_with_prices = add_initial_category_prices_to_shipments(
        shipments_with_prices=shipments_with_prices,
        prices_propositions_by_category_df=prices_propositions_by_category_df,
    )

    return sum_multiple_billings_prices(shipments_with_prices=shipments_with_prices)



import pandas as pd


def select_column_of_most_recent_entity(df_entity) -> pd.Series:
    return df_entity.sort_values(by="created_at", ascending=False).iloc[0:1]


def select_best_proposition_id(shipment_propositions: pd.DataFrame):
    mask_accepted = (shipment_propositions["proposition_status"] == "accepted") | (
        shipment_propositions["proposition_status"] == "accepted_old_version"
    )
    mask_sent = shipment_propositions["proposition_status"] == "sent"
    mask_purchase_ready = shipment_propositions["proposition_status"] == "purchase_ready"

    accepted_propositions = shipment_propositions[mask_accepted]
    if len(accepted_propositions) > 0:
        return select_column_of_most_recent_entity(accepted_propositions)
    sent_propositions = shipment_propositions[mask_sent]
    if len(sent_propositions) > 0:
        return select_column_of_most_recent_entity(sent_propositions)
    purchase_ready_propositions = shipment_propositions[mask_purchase_ready]
    if len(purchase_ready_propositions) > 0:
        return select_column_of_most_recent_entity(purchase_ready_propositions)
    if len(shipment_propositions) > 0:
        return select_column_of_most_recent_entity(shipment_propositions)


def match_shipments_propositions(shipments_df: pd.DataFrame, propositions_df: pd.DataFrame):
    propositions_df = propositions_df.rename(columns={"id": "proposition_id", "status": "proposition_status"})
    propositions_df = propositions_df.groupby("shipment_id").apply(select_best_proposition_id).reset_index(drop=True)
    shipments_df = shipments_df.rename(columns={"id": "shipment_id", "status": "shipment_status"})
    return pd.merge(
        shipments_df.drop(["transit_time"], axis=1),
        propositions_df[
            [
                "proposition_id",
                "proposition_status",
                "purchase_ready_date",
                "sent_date",
                "accepted_date",
                "shipment_id",
                "transit_time",
            ]
        ],
        how="left",
        on="shipment_id",
    )



import pandas as pd


def add_shipments_to_cq(computed_quotations_df: pd.DataFrame, shipments_all_propositions_df: pd.DataFrame):
    computed_quotations_df = computed_quotations_df.rename(columns={"id": "computed_quotation_id"})
    computed_quotations_df = computed_quotations_df.dropna(subset=["proposition_id"])
    computed_quotations_df = computed_quotations_df.sort_values(by=["proposition_id", "created_at"], ascending=False)
    computed_quotations_df = computed_quotations_df.drop_duplicates(subset=["proposition_id"])
    computed_quotations_df = pd.merge(
        shipments_all_propositions_df[["shipment_id", "proposition_id"]],
        computed_quotations_df.drop(["shipment_id"], axis=1),
        how="inner",
        on="proposition_id",
    )
    return computed_quotations_df
